# Configure paths for ASSA
dnl ----------------------------------------------------------------------------
dnl Name: assa.m4
dnl Desc: m4 libassa version test macro
dnl Date: Wed Aug 21 23:18:37 EDT 2002
dnl Note: Borrowed mostly from gtkmm--.m4
dnl ----------------------------------------------------------------------------
dnl
dnl Test for ASSA, and define ASSA_CFLAGS and ASSA_LIBS
dnl to be used as follows:
dnl
dnl AM_PATH_ASSA([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl
dnl For example, you migth add something like this to your project's
dnl ${projroot}/configure.in:
dnl
dnl -"--"--"--"--"--"--"--"--"--"--"--"--"--"--"--
dnl AM_PATH_ASSA(1.4.1,,AC_MSG_ERROR([*** libassa >= 1.4 not installed ***]))
dnl 
dnl AC_SUBST(ASSA_CFLAGS)
dnl AC_SUBST(ASSA_LIBS)
dnl -"--"--"--"--"--"--"--"--"--"--"--"--"--"--"--
dnl
dnl This tests for the installed library version be at least 1.4.0, and
dnl makes ASSA_CFLAGS and ASSA_LIBS available for use in any Makefile.am
dnl in your project. For example, my killerapp's Makefile.am might look like 
dnl this:
dnl
dnl -"--"--"--"--"--"--"--"--"--"--"--"--"--"--"--
dnl bin_PROGRAMS = killerapp
dnl
dnl CXXFLAGS = -g @ASSA_CFLAGS@ ...
dnl
dnl killerapp_LDADD = @ASSA_LIBS@ @LIBS@ ...
dnl -"--"--"--"--"--"--"--"--"--"--"--"--"--"--"--
dnl
dnl ----------------------------------------------------------------------------

AC_DEFUN(AM_PATH_ASSA,
[dnl 
dnl Get the cflags and libraries from the assa-config script
dnl
AC_ARG_WITH(assa-prefix,[  --with-assa-prefix=PREFIX
                          Prefix where ASSA is installed (optional)],
            assa_config_prefix="$withval", assa_config_prefix="")
AC_ARG_WITH(assa-exec-prefix,[  --with-assa-exec-prefix=PREFIX
                          Exec prefix where ASSA is installed (optional)],
            assa_config_exec_prefix="$withval", assa_config_exec_prefix="")
AC_ARG_ENABLE(assatest, [  --disable-assatest    
	                      Do not try to compile and run a test ASSA program],
		    , enable_assatest=yes)

  if test x$assa_config_exec_prefix != x ; then
     assa_config_args="$assa_config_args --exec-prefix=$assa_config_exec_prefix"
     if test x${ASSA_CONFIG+set} != xset ; then
        ASSA_CONFIG=$assa_config_exec_prefix/bin/assa-config
     fi
  fi
  if test x$assa_config_prefix != x ; then
     assa_config_args="$assa_config_args --prefix=$assa_config_prefix"
     if test x${ASSA_CONFIG+set} != xset ; then
        ASSA_CONFIG=$assa_config_prefix/bin/assa-config
     fi
  fi

  AC_PATH_PROG(ASSA_CONFIG, assa-config, no)

dnl $1 = MINIMUM VERSION
dnl if ($1 == "") 
dnl     min_assa_version = 1.4.0
dnl else
dnl     min_assa_version = $1
dnl
  min_assa_version=ifelse([$1], ,1.4.0,$1)

  AC_MSG_CHECKING(for ASSA - version >= $min_assa_version)
  AC_LANG_SAVE
  no_assa=""
  if test "$ASSA_CONFIG" = "no" ; then
    no_assa=yes
  else
    AC_LANG_CPLUSPLUS

    ASSA_CFLAGS=`$ASSA_CONFIG $assa_config_args --cflags`
    ASSA_LIBS=`$ASSA_CONFIG $assa_config_args --libs`
    assa_config_major_version=`$ASSA_CONFIG $assa_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    assa_config_minor_version=`$ASSA_CONFIG $assa_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    assa_config_micro_version=`$ASSA_CONFIG $assa_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    if test "x$enable_assatest" = "xyes" ; then
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CXXFLAGS="$CXXFLAGS $ASSA_CFLAGS"
      LIBS="$LIBS $ASSA_LIBS"
dnl
dnl Now check if the installed libassa is sufficiently new. (Also sanity
dnl checks the results of assa-config to some extent
dnl
      rm -f conf.assatest
      AC_TRY_RUN([
#include <assa/Assa.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int 
main ()
{
  int major, minor, micro;
  char *tmp_version = NULL;
  major = minor = micro = 0;

  system ("touch conf.assatest");

  tmp_version = strdup("$min_assa_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_assa_version");
     exit(1);
   }

	if (($assa_config_major_version > major) 
        ||
        (($assa_config_major_version == major) && 
         ($assa_config_minor_version > minor)) 
        ||
        (($assa_config_major_version == major) && 
         ($assa_config_minor_version == minor) && 
         ($assa_config_micro_version >= micro)))
      {
        return 0;
       }
     else
      {
        printf("\n*** An old version of libassa (%d.%d.%d) was found.\n",
               $assa_config_major_version, 
			   $assa_config_minor_version, 
			   $assa_config_micro_version);
        printf("*** You need a version of GTK-- newer than %d.%d.%d.\n");
        printf("***\n");
        printf("*** If you have already installed a sufficiently new version, this error\n");
        printf("*** probably means that the wrong copy of the assa-config shell script is\n");
        printf("*** being found. The easiest way to fix this is to remove the old version\n");
        printf("*** of libassa, but you can also set the ASSA_CONFIG environment to point to the\n");
        printf("*** correct copy of assa-config. (In this case, you will have to\n");
        printf("*** modify your LD_LIBRARY_PATH enviroment variable, or edit /etc/ld.so.conf\n");
        printf("*** so that the correct libraries are found at run-time))\n");
      }
  return 1;
}
],, no_assa=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CXXFLAGS="$ac_save_CXXFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_assa" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$ASSA_CONFIG" = "no" ; then
       echo "*** The assa-config script installed by libassa could not be found"
       echo "*** If libassa was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the ASSA_CONFIG environment variable to the"
       echo "*** full path to assa-config."
     else
       if test -f conf.assatest ; then
        :
       else
          echo "*** Could not run libassa test program, checking why..."
          CXXFLAGS="$CXXFLAGS $ASSA_CFLAGS"
          LIBS="$LIBS $ASSA_LIBS"
          AC_TRY_LINK([
#include <assa/Assa.h>
#include <stdio.h>
],      [ return ((assa_major_version) || (assa_minor_version) || (assa_micro_version)); ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding libassa or finding the wrong"
          echo "*** version of libassa. If it is not finding libassa, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH" ],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means libassa was incorrectly installed"
          echo "*** or that you have moved libassa since it was installed. In the latter case, you"
          echo "*** may want to edit the assa-config script: $ASSA_CONFIG" ])
          CXXFLAGS="$ac_save_CXXFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     ASSA_CFLAGS=""
     ASSA_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_LANG_RESTORE
  AC_SUBST(ASSA_CFLAGS)
  AC_SUBST(ASSA_LIBS)
  rm -f conf.assatest
])

